package com.zoibackgroundlocation;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.widget.AppCompatCheckBox;

import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.module.annotations.ReactModule;
import com.facebook.react.modules.core.DeviceEventManagerModule;
import com.google.gson.Gson;

import java.util.Arrays;
import java.util.List;


@ReactModule(name = ZoiBackgroundlocationModule.NAME)
public class ZoiBackgroundlocationModule extends ReactContextBaseJavaModule {
  public static final String NAME = "ZoiBackgroundlocation";

  private static ReactApplicationContext reactContext;

  TimerReceiver receiver;

  // LocationService mLocationService = new LocationService();
  // Intent mServiceIntent;

  public ZoiBackgroundlocationModule(ReactApplicationContext context) {

    super(context);

    reactContext = context;

    receiver = new TimerReceiver();

    context.registerReceiver(receiver, new IntentFilter("GET_LOCATION"));
  }

  @Override
  @NonNull
  public String getName() {
    return NAME;
  }


  // Example method
  // See https://reactnative.dev/docs/native-modules-android
  @ReactMethod
  public void multiply(double a, double b, Promise promise) {
    promise.resolve(a * b);


  }

  @ReactMethod
  public void start(Promise response) {
    try {

      Activity currentActivity = getCurrentActivity();

      LocationService mLocationService = new LocationService();

       startPowerSaverIntent(reactContext,currentActivity);

      if (!Util.isMyServiceRunning(mLocationService.getClass(), currentActivity)) {
        Intent mServiceIntent;

        mServiceIntent = new Intent(currentActivity.getBaseContext(), mLocationService.getClass());
        // mServiceIntent.putExtra("url","https://ms.zoiftspl.com/sdlapi");
        // mServiceIntent.putExtra("id","11111");

        if (!Util.isMyServiceRunning(mLocationService.getClass(), currentActivity)) {

          currentActivity.getBaseContext().startService(mServiceIntent);


        } else {


        }

      }
    } catch (Exception e) {
      response.reject("Error", e);
    }


  }


  class TimerReceiver extends BroadcastReceiver {

    @Override
    public void onReceive(Context context, Intent intent) {

      if (intent.getAction().equals("GET_LOCATION")) {

        Bundle extras = intent.getExtras();
        if (extras == null) {
          Log.d("Service", "null");
        } else {
          String latitude = (String) extras.get("Latitude");
          String longitude = (String) extras.get("Longitude");

          Gson gson = new Gson();
          LocationDetails loc = new LocationDetails();
          loc.Latitude = latitude;
          loc.Longitude = longitude;

          String jsonString = gson.toJson(loc);

          Log.d("jsonString", gson.toJson(loc));

          Toast.makeText(context, gson.toJson(loc), Toast.LENGTH_LONG).show();

          reactContext
            .getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
            .emit("onSessionConnect", gson.toJson(loc));
        }
      }

    }

  }

  class LocationDetails {
    private String Latitude;
    private String Longitude;
  }

  public static List<Intent> POWER_MANAGER_INTENTS = Arrays.asList(
    new Intent().setComponent(new ComponentName("com.miui.securitycenter", "com.miui.permcenter.autostart.AutoStartManagementActivity")),
    new Intent().setComponent(new ComponentName("com.letv.android.letvsafe", "com.letv.android.letvsafe.AutobootManageActivity")),
    new Intent().setComponent(new ComponentName("com.huawei.systemmanager", "com.huawei.systemmanager.optimize.process.ProtectActivity")),
    new Intent().setComponent(new ComponentName("com.coloros.safecenter", "com.coloros.safecenter.permission.startup.StartupAppListActivity")),
    new Intent().setComponent(new ComponentName("com.coloros.safecenter", "com.coloros.safecenter.startupapp.StartupAppListActivity")),
    new Intent().setComponent(new ComponentName("com.oppo.safe", "com.oppo.safe.permission.startup.StartupAppListActivity")),
    new Intent().setComponent(new ComponentName("com.iqoo.secure", "com.iqoo.secure.ui.phoneoptimize.AddWhiteListActivity")),
    new Intent().setComponent(new ComponentName("com.iqoo.secure", "com.iqoo.secure.ui.phoneoptimize.BgStartUpManager")),
    new Intent().setComponent(new ComponentName("com.vivo.permissionmanager", "com.vivo.permissionmanager.activity.BgStartUpManagerActivity")),
    new Intent().setComponent(new ComponentName("com.asus.mobilemanager", "com.asus.mobilemanager.entry.FunctionActivity")),
    new Intent().setComponent(new ComponentName("com.asus.mobilemanager", "com.asus.mobilemanager.entry.FunctionActivity")),
    new Intent().setComponent(new ComponentName("com.asus.mobilemanager", "com.asus.mobilemanager.autostart.AutoStartActivity")),
    new Intent().setComponent(new ComponentName("com.letv.android.letvsafe", "com.letv.android.letvsafe.AutobootManageActivity"))
      .setData(android.net.Uri.parse("mobilemanager://function/entry/AutoStart")),
    new Intent().setComponent(new ComponentName("com.meizu.safe", "com.meizu.safe.security.SHOW_APPSEC")).addCategory(Intent.CATEGORY_DEFAULT).putExtra("packageName", "com.sdl")

  );


  public static void startPowerSaverIntent(Context context, Activity activity) {


    SharedPreferences settings = context.getSharedPreferences("ProtectedApps", Context.MODE_PRIVATE);
    boolean skipMessage = settings.getBoolean("skipProtectedAppCheck", false);

    Log.d("skipProtectedAppCheck", String.valueOf(skipMessage));
    if (!skipMessage) {
      final SharedPreferences.Editor editor = settings.edit();
      boolean foundCorrectIntent = false;
      for (Intent intent : POWER_MANAGER_INTENTS) {

        Log.d("intent", String.valueOf(intent));
        if (isCallable(context, intent)) {
          foundCorrectIntent = true;
          final AppCompatCheckBox dontShowAgain = new AppCompatCheckBox(context);
          // dontShowAgain.setText("Do not show again");
          dontShowAgain.setOnCheckedChangeListener((buttonView, isChecked) -> {
            editor.putBoolean("skipProtectedAppCheck", isChecked);
            editor.apply();
          });

          intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);

          new AlertDialog.Builder(activity)
            .setTitle(Build.MANUFACTURER + " Protected Apps")
            .setMessage(String.format("%s requires to be enabled in 'Protected Apps' to function properly.%n", "SDL"))
            .setView(dontShowAgain)
            .setPositiveButton("Go to settings", (dialog, which) -> context.startActivity(intent))
            .setNegativeButton(android.R.string.cancel, null)
            .show();
          break;
        }
      }
      if (!foundCorrectIntent) {
        editor.putBoolean("skipProtectedAppCheck", true);
        editor.apply();
      }
    }
  }


  private static boolean isCallable(Context context, Intent intent) {
    List<ResolveInfo> list = context.getPackageManager().queryIntentActivities(intent,
      PackageManager.MATCH_DEFAULT_ONLY);
    return list.size() > 0;
  }

}
